<?php
/**
 * @file
 * file containing functions related to nodes processing.
 */

/**
 * Record transformation process.
 *
 * Create a new node or update it if existing.
 *
 * @param array $record
 *   Containing record values.
 *
 * @param array $mapping
 *   Which record fields should be mapped with which node fields.
 *
 * @param object $node
 *   An existing node for the record.
 *
 * @return object
 *   An new or updated node.
 */
function opac_transform_to_node($record, $mapping, $node = NULL) {
  // Means we got a new record. So create a new node.
  if (!$node) {
    $node = opac_create_node($record['opac_content_type']);
  }

  if ($node->type != $record['opac_content_type']) {
    node_delete($node->nid);
    $node = opac_create_node($record['opac_content_type']);

    // Log this.
    $message = "Content type od record n° %recordid coming from %servid server has changed.";
    $message .= " Title: %title. Switch this record";
    $variables = array(
      '%recordid' => $id,
      '%servid' => $server->serv_id,
      '%title' => $bib['title'],
    );
    watchdog('OPAC HARVEST', $message, $variables, WATCHDOG_NOTICE);
  }

  // Apply the mapping to the node. Means that
  // mapped fields are switched from record to the node.
  opac_record_to_node_fields($record, $node, $mapping);

  // We don't want an opac node to be promoted
  // to the front page.
  // May be opac module should provide a rules maker
  // to define if a node should be promoted?
  $node->promote = 0;
  return $node;
}

/**
 * Apply a mapping to a node.
 *
 * @param array $record
 *   Containing record values.
 *
 * @param object $node
 *   An existing node for the record.
 *
 * @param array $mappings
 *   Which record fields should be mapped with which node fields.
 */
function opac_record_to_node_fields($record, &$node, $mappings) {
  module_load_include('inc', 'opac', 'includes/opac.plugin');
  $node->title = $record['title'];

  // Loop through all mapped fields.
  // All values in remotefield fields will
  // be copied in nodefield.
  foreach ($mappings['mapped'] as $mapping) {
    $nodefield = $mapping['node_field_name'];
    $node->{$nodefield}['und'] = array();

    $plugin = opac_plugin_get($mapping['plugin']);
    opac_plugin_require_file($plugin);
    if ($plugin && function_exists($plugin['value callback'])) {
      $node->{$nodefield} = $plugin['value callback']($record, $mapping);
    }
    else {
      $field_info = field_info_field($nodefield);
      if (isset($field_info['columns']['value'])) {
        $remotefield = $mapping['mapped_with'];
        if (isset($record[$remotefield])) {
          foreach ($record[$remotefield] as $value) {
            $node->{$nodefield}['und'][] = array('value' => $value);
          }
        }
      }
      else {
        $message = "Field '@field' does not contain a 'value' column. "
          . "Please consider using a plugin for this field.";
        $variables = array('@field' => $nodefield);
        watchdog('opac', $message, $variables, WATCHDOG_WARNING);
      }
    }

  }
}

/**
 * Create a new node.
 *
 * @return object
 *   A new node.
 */
function opac_create_node($type) {
  $node = new stdClass();
  $node->type = $type;
  node_object_prepare($node);
  $node->uid = 1;
  $node->comment = 0;
  $node->language = 'und';
  return $node;
}

/**
 * Check if a node exists.
 *
 * Search node whose opac_server_id_field and
 * opac_record_id_field match serv_id and record id parameters.
 *
 * @param string $serv_id
 *   ILS server identifier.
 *
 * @param string $record_id
 *   Record identifier.
 *
 * @return object
 *   The matching node or NULL.
 */
function opac_existing_node($serv_id, $record_id) {
  // In order to get an existing node, we have to
  // search it by checking server and record identifier.
  // This two fields being configurable, we take them
  // with opac_server_id_field and opac_record_id_field
  // Drupal variables.
  $server_id_field = variable_get('opac_server_id_field', 'none');
  $record_id_field = variable_get('opac_record_id_field', 'none');

  $content_types = array_keys(opac_get_content_types($serv_id));
  // Prepare an EntityFieldQuery.
  $query = new EntityFieldQuery();

  $query->entityCondition('entity_type', 'node', '=')
    ->propertyCondition('type', $content_types, 'IN')
    ->fieldCondition($record_id_field, 'value', $record_id, '=')
    ->fieldCondition($server_id_field, 'value', $serv_id, '=');

  // Process the query.
  $entities = $query->execute();
  if (count($entities)) {
    $nodes_id = array_keys($entities['node']);
    return node_load($nodes_id[0]);
  }

  return NULL;
}

/**
 * Build an existing nodes list.
 *
 * @return array
 *   All the opac nodes with nid, server id and biblio id.
 */
function opac_get_nodes() {
  $server_id_field = variable_get('opac_server_id_field', 'none');
  $record_id_field = variable_get('opac_record_id_field', 'none');
  // Prepare an EntityFieldQuery.
  $query = new EntityFieldQuery();

  echo "Retrieving existing nodes\n";
  $query->entityCondition('entity_type', 'node', '=');

  $result = $query->execute();
  $entities = array();
  if (isset($result['node'])) {
    $entities = entity_load('node', array_keys($result['node']));
  }

  $opac_nodes = array();
  foreach ($entities as $nid => $entity) {
    $recordid = isset($entity->$record_id_field) ? $entity->{$record_id_field}['und'][0]['value'] : NULL;
    $serverid = isset($entity->$server_id_field) ? $entity->{$server_id_field}['und'][0]['value'] : NULL;
    if (isset($recordid) && isset($serverid)) {
      $opac_nodes[$serverid][$recordid] = $nid;
    }
  }
  echo "Ok\n";
  return $opac_nodes;
}
